package info.bukova.isspst.security;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.services.users.RoleService;
import info.bukova.isspst.services.users.UserService;

import java.util.Collection;

import javax.naming.NamingException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.ldap.core.DirContextOperations;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.ldap.userdetails.LdapAuthoritiesPopulator;

public class AuthPopulator implements LdapAuthoritiesPopulator {

	private UserService userService;
	private RoleService roleService;

	public AuthPopulator(UserService userService, RoleService roleService) {
		this.userService = userService;
		this.roleService = roleService;
	}

	@Override
	public Collection<? extends GrantedAuthority> getGrantedAuthorities(
			DirContextOperations userData, String login) {

		User user = null;
		try {
			user = (User) userService.loadUserByUsername(login);
		} catch (UsernameNotFoundException e) {
			Logger logger = LoggerFactory.getLogger(AuthPopulator.class);
			logger.info("Importing user from LDAP");
			
			user = new User();
			user.setUsername(login);
			Role role = roleService.getRoleByAuthority(Constants.ROLE_USER);
			user.addAuthority(role);
			
			if (userData.attributeExists("givenName")) {
				try {
					user.setFirstName(userData.getAttributes().get("givenName").get().toString());
				} catch (NamingException e1) {
					logger.info("LDAP object has no 'givenName' attribute");
				}
			}
			if (userData.attributeExists("sn")) {
				try {
					user.setLastName(userData.getAttributes().get("sn").get().toString());
				} catch (NamingException e1) {
					logger.info("LDAP object has no 'sn' attribute");
				}
			}
			if (userData.attributeExists("mail")) {
				try {
					user.setEmail(userData.getAttributes().get("mail").get().toString());
				} catch (NamingException e1) {
					logger.info("LDAP object has no 'mail' attribute");
				}
			}
			
			userService.grantAdmin();
			userService.add(user);
			userService.removeAccess();
		}

		return user != null ? user.getAuthorities() : null;
	}

}
