package info.bukova.isspst.ui.settings;

import info.bukova.isspst.data.Address;
import info.bukova.isspst.data.Requirement;
import info.bukova.isspst.data.SettingsData;
import info.bukova.isspst.data.Vehicle;
import info.bukova.isspst.mail.MailMessage;
import info.bukova.isspst.services.settings.GlobalSettingsService;
import info.bukova.isspst.sort.ReflectionTools;
import info.bukova.isspst.ui.LocaleConverter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.zkoss.bind.annotation.BindingParam;
import org.zkoss.bind.annotation.Command;
import org.zkoss.bind.annotation.Init;
import org.zkoss.bind.annotation.NotifyChange;
import org.zkoss.zk.ui.select.annotation.WireVariable;
import org.zkoss.zul.Window;

public class GlobalSettingsVM {
	
	@WireVariable
	private GlobalSettingsService settingsService;
	private SettingsData settings;
	private LocaleConverter locConverter;
	private Address selAddress;
	private List<Integer> refundsHours;
	
	@Init
	public void init() {
		settings = settingsService.getSettings();
		locConverter = new LocaleConverter();
		setRefundsHours(settings.getRefunds().keySet());
	}

	public SettingsData getSettings() {
		return settings;
	}
	
	@Command
	public void save(@BindingParam("window") Window window) {
		settingsService.updateSettings();
		window.detach();
	}
	
	public List<String> getRequirementFields() {
		return ReflectionTools.getEntityFields(Requirement.class);
	}
	
	public boolean isCanSave() {
		return true;
	}

	public LocaleConverter getLocConverter() {
		return locConverter;
	}
	
	@Command
	@NotifyChange("settings")
	public void insertField(@BindingParam("field") String field, @BindingParam("message") MailMessage message) {
		message.setText(message.getText() + "[" + field + "]");
	}
	
	@Command
	@NotifyChange("settings")
	public void addAddress() {
		settings.getShippingAddrs().add(new Address());
	}
	
	@Command
	@NotifyChange("settings")
	public void removeAddress(@BindingParam("addr") Address addr) {
		settings.getShippingAddrs().remove(addr);
	}
	
	@Command
	@NotifyChange("settings")
	public void addVehicle() {
		settings.getVehicles().add(new Vehicle());
	}
	
	@Command
	@NotifyChange("settings")
	public void removeVehicle(@BindingParam("vehicle") Vehicle vehicle) {
		settings.getVehicles().remove(vehicle);
	}

	public Address getSelAddress() {
		return selAddress;
	}

	public void setSelAddress(Address selAddress) {
		this.selAddress = selAddress;
	}

	public List<Integer> getRefundsHours() {
		return refundsHours;
	}

	private void setRefundsHours(Set<Integer> refundsHours) {
		this.refundsHours = new ArrayList<Integer>(settings.getRefunds().keySet()); 
		Collections.sort(this.refundsHours);
	}

}
